%define anolis_release 1

# The source directory.
%global source_directory 1.20-stable

Name:           libnbd
Version:        1.20.3
Release:        %{anolis_release}%{?dist}
Summary:        NBD client library in userspace

License:        LGPL-2.0-or-later AND BSD-3-Clause
URL:            https://gitlab.com/nbdkit/libnbd

Source0:        http://libguestfs.org/download/libnbd/%{source_directory}/%{name}-%{version}.tar.gz
Patch0:		libnbd-1.18.0-linkage.patch

# https://gitlab.com/nbdkit/libnbd/-/commit/cf49a49adc8abc8c917437db7461ed9956583877
Patch0001:     0001-generator-Print-full-error-in-handle_reply_error.patch
# https://gitlab.com/nbdkit/libnbd/-/commit/474a4ae6c8d11212a4a8c06ea3e8b3fd97a7e97d 
Patch0002:     0002-lib-Don-t-overwrite-error-in-nbd_opt_-go-info.patch
# https://gitlab.com/nbdkit/libnbd/-/commit/e75d20b9e19143b1bd0d232fc49cb2e0287f824a 
Patch0003:     0003-generator-Restore-assignment-to-local-err.patch
# https://gitlab.com/nbdkit/libnbd/-/commit/5dbfc418cb6176102634acea2256b2335520159c 
Patch0004:     0004-generator-states-newstyle.c-Quote-untrusted-string-f.patch
# https://gitlab.com/nbdkit/libnbd/-/commit/0d6c6bbb3386de3b60ab6c4831045f2b1896051b 
Patch0005:     0005-generator-states-newstyle.c-Don-t-sign-extend-escape.patch

# For the core library.
BuildRequires:  gcc
BuildRequires:  make
BuildRequires:  /usr/bin/pod2man
BuildRequires:  gnutls-devel
BuildRequires:  libxml2-devel

# For nbdfuse.
BuildRequires:  fuse3, fuse3-devel

# For the Python 3 bindings.
BuildRequires:  python3-devel

# Only for building the examples.
BuildRequires:  glib2-devel

# For bash-completion.
BuildRequires:  bash-completion

# Only for running the test suite.
BuildRequires:  coreutils
BuildRequires:  gcc-c++
BuildRequires:  gnutls-utils
BuildRequires:  iproute
BuildRequires:  jq
BuildRequires:  util-linux

%description
NBD — Network Block Device — is a protocol for accessing Block Devices
(hard disks and disk-like things) over a Network.

This is the NBD client library in userspace, a simple library for
writing NBD clients.

The key features are:

 * Synchronous and asynchronous APIs, both for ease of use and for
   writing non-blocking, multithreaded clients.

 * High performance.

 * Minimal dependencies for the basic library.

 * Well-documented, stable API.

 * Bindings in several programming languages.


%package devel
Summary:        Development headers for %{name}
Requires:       %{name} = %{version}-%{release}


%description devel
This package contains development headers for %{name}.


%package -n python3-%{name}
Summary:        Python 3 bindings for %{name}
Requires:       %{name} = %{version}-%{release}
%{?python_provide:%python_provide python3-%{name}}

# The Python module happens to be called lib*.so.  Don't scan it and
# have a bogus "Provides: libnbdmod.*".
%global __provides_exclude_from ^%{python3_sitearch}/lib.*\\.so


%description -n python3-%{name}
python3-%{name} contains Python 3 bindings for %{name}.


%package -n nbdfuse
Summary:        FUSE support for %{name}
Requires:       %{name} = %{version}-%{release}
Recommends:     fuse3


%description -n nbdfuse
This package contains FUSE support for %{name}.


%package bash-completion
Summary:       Bash tab-completion for %{name}
BuildArch:     noarch
Requires:      %{name} = %{version}-%{release}
Supplements:   (%{name} and bash-completion)


%description bash-completion
Install this package if you want intelligent bash tab-completion
for %{name}.


%prep
%autosetup -p1


%build
%configure \
    --disable-static \
    --with-tls-priority=@LIBNBD,SYSTEM \
    PYTHON=%{__python3} \
    --enable-python \
    --disable-ocaml \
    --enable-fuse \
    --disable-golang \
    --disable-rust

%make_build


%install
%make_install

# Delete the golang man page since we're not distributing the bindings.
rm $RPM_BUILD_ROOT%{_mandir}/man3/libnbd-golang.3*

# Delete the OCaml man page
rm $RPM_BUILD_ROOT%{_mandir}/man3/libnbd-ocaml.3*

%{generate_compatibility_deps}

%check
function skip_test ()
{
    for f in "$@"; do
        rm -f "$f"
        echo 'exit 77' > "$f"
        chmod +x "$f"
    done
}

# All fuse tests fail in Koji with:
# fusermount: entry for fuse/test-*.d not found in /etc/mtab
# for unknown reasons but probably related to the Koji environment.
skip_test fuse/test-*.sh

# IPv6 loopback connections fail in Koji.
make -C tests connect-tcp6 ||:
skip_test tests/connect-tcp6

%make_build check || {
    for f in $(find -name test-suite.log); do
        echo
        echo "==== $f ===="
        cat $f
    done
    exit 1
  }


%files
%license COPYING.LIB
%{_bindir}/nbdcopy
%{_bindir}/nbddump
%{_bindir}/nbdinfo
%{_libdir}/libnbd.so.*
%{_mandir}/man1/nbdcopy.1*
%{_mandir}/man1/nbddump.1*
%{_mandir}/man1/nbdinfo.1*
%dir %{abidir}
%{abidir}/*.dump
%{abidir}/*-option.list
%exclude %{abidir}/nbdfuse-option.list

%files devel
%license examples/LICENSE-FOR-EXAMPLES
%{_includedir}/libnbd.h
%{_libdir}/libnbd.so
%{_libdir}/pkgconfig/libnbd.pc
%{_mandir}/man3/libnbd.3*
%{_mandir}/man1/libnbd-release-notes-1.*.1*
%{_mandir}/man3/libnbd-security.3*
%{_mandir}/man3/nbd_*.3*

%files -n python3-%{name}
%{python3_sitearch}/libnbdmod*.so
%{python3_sitearch}/nbd.py
%{python3_sitearch}/nbdsh.py
%{python3_sitearch}/__pycache__/nbd*.py*
%{_bindir}/nbdsh
%{_mandir}/man1/nbdsh.1*


%files -n nbdfuse
%{_bindir}/nbdfuse
%{abidir}/nbdfuse-option.list
%{_mandir}/man1/nbdfuse.1*

%files bash-completion
%dir %{_datadir}/bash-completion/completions
%{_datadir}/bash-completion/completions/nbdcopy
%{_datadir}/bash-completion/completions/nbddump
%{_datadir}/bash-completion/completions/nbdfuse
%{_datadir}/bash-completion/completions/nbdinfo
%{_datadir}/bash-completion/completions/nbdsh

%changelog
* Tue Feb 18 2025 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.20.3-1
- Update to 1.20.3 from 1.18.5
- Add patches to print the full error from the server during handshaking. 
  [Patches cherry-pick from c10s. Original changelog: 
  - Print full NBD error from server.]

* Tue Jan 14 2025 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.18.5-1
- to #IBHRLO
- update to 1.18.5 for CVE-2024-7383
- Project: TC2024080204
- Signed-off-by: mgb01105731 mgb01105731@alibaba-inc.com

* Tue Mar 12 2024 Bo Ren <rb01097748@alibaba-inc.com> - 1.18.1-2
- Rebuild with python3.11

* Wed Oct 04 2023 Funda Wang <fundawang@yeah.net> - 1.18.1-1
- Import package
